package com.jpattern.orm.test.crud;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import org.junit.Test;

import com.jpattern.orm.BaseTestShared;
import com.jpattern.orm.classmapper.ClassMapperBuilder;
import com.jpattern.orm.classmapper.ClassTableMapBuilder;
import com.jpattern.orm.classmapper.IClassMapper;
import com.jpattern.orm.classmapper.IClassMapperBuilder;
import com.jpattern.orm.classmapper.TableMap;
import com.jpattern.orm.crud.IOrmCRUDQuery;
import com.jpattern.orm.crud.IOrmCRUDQueryGenerator;
import com.jpattern.orm.crud.OrmCRUDQueryGenerator;
import com.jpattern.orm.dialect.DefaultDialect;
import com.jpattern.orm.session.SessionProvider;
import com.jpattern.orm.test.domain.Employee;
import com.jpattern.orm.test.domain.People;
import com.jpattern.orm.test.domain.Zoo_People;

/**
 * 
 * @author Francesco Cina
 *
 * 01/giu/2011
 */
public class OrmCRUDQueryGeneratorTest extends BaseTestShared {

	@Override
	protected void setUp() throws Exception {
	}

	@Override
	protected void tearDown() throws Exception {
	}

	@Test
	public void testOrmProvider() throws Exception {
		final SessionProvider connectionProvider = this.getJPOrm().getSessionProvider();
		this.ormCrudWithSequence(connectionProvider);
		this.ormCrud(connectionProvider);
		this.ormCrudWithSchema(connectionProvider);
		this.ormCrudSelectFromClause(connectionProvider);
	}

	public void ormCrudWithSequence(SessionProvider sessionProvider) throws Exception {
		final Class<People> clazz = People.class;
		final TableMap<People> tableMap = new TableMap<People>();
		new ClassTableMapBuilder<People>(clazz, tableMap).updateTableMap();
		final IClassMapperBuilder<People> classMapperGenerator = new ClassMapperBuilder<People>(clazz, tableMap);
		final IClassMapper<People> classMapper = classMapperGenerator.generate();
		assertNotNull(classMapper);

		final IOrmCRUDQueryGenerator<People> ormGenerator = new OrmCRUDQueryGenerator<People>(new DefaultDialect(), classMapper);
		final IOrmCRUDQuery ormCrudQuery = ormGenerator.generate();
		assertNotNull(ormCrudQuery);

		System.out.println(ormCrudQuery.getLoadQuery());
		System.out.println(ormCrudQuery.getUpdateQuery());
		System.out.println(ormCrudQuery.getDeleteQuery());
		System.out.println(ormCrudQuery.getSaveQuery());
	}

	public void ormCrud(SessionProvider sessionProvider) throws Exception {
		final Class<Employee> clazz = Employee.class;
		final TableMap<Employee> tableMap = new TableMap<Employee>();
		new ClassTableMapBuilder<Employee>(clazz, tableMap).updateTableMap();
		final IClassMapperBuilder<Employee> classMapperGenerator = new ClassMapperBuilder<Employee>(clazz, tableMap);
		final IClassMapper<Employee> classMapper = classMapperGenerator.generate();
		assertNotNull(classMapper);

		final IOrmCRUDQueryGenerator<Employee> ormGenerator = new OrmCRUDQueryGenerator<Employee>(new DefaultDialect(), classMapper);
		final IOrmCRUDQuery ormCrudQuery = ormGenerator.generate();
		assertNotNull(ormCrudQuery);

		System.out.println(ormCrudQuery.getLoadQuery());
		System.out.println(ormCrudQuery.getUpdateQuery());
		System.out.println(ormCrudQuery.getDeleteQuery());
		System.out.println(ormCrudQuery.getSaveQuery());
	}

	public void ormCrudWithSchema(SessionProvider sessionProvider) throws Exception {
		final Class<Zoo_People> clazz = Zoo_People.class;
		final TableMap<Zoo_People> tableMap = new TableMap<Zoo_People>();
		new ClassTableMapBuilder<Zoo_People>(clazz, tableMap).updateTableMap();
		final IClassMapperBuilder<Zoo_People> classMapperGenerator = new ClassMapperBuilder<Zoo_People>(clazz, tableMap);
		final IClassMapper<Zoo_People> classMapper = classMapperGenerator.generate();
		assertNotNull(classMapper);

		final IOrmCRUDQueryGenerator<Zoo_People> ormGenerator = new OrmCRUDQueryGenerator<Zoo_People>(new DefaultDialect(), classMapper);
		final IOrmCRUDQuery ormCrudQuery = ormGenerator.generate();
		assertNotNull(ormCrudQuery);

		System.out.println(ormCrudQuery.getLoadQuery());
		System.out.println(ormCrudQuery.getUpdateQuery());
		System.out.println(ormCrudQuery.getDeleteQuery());
		System.out.println(ormCrudQuery.getSaveQuery());
	}

	public void ormCrudSelectFromClause(SessionProvider sessionProvider) throws Exception {
		final Class<Zoo_People> clazz = Zoo_People.class;
		final TableMap<Zoo_People> tableMap = new TableMap<Zoo_People>();
		new ClassTableMapBuilder<Zoo_People>(clazz, tableMap).updateTableMap();
		final IClassMapperBuilder<Zoo_People> classMapperGenerator = new ClassMapperBuilder<Zoo_People>(clazz, tableMap);
		final IClassMapper<Zoo_People> classMapper = classMapperGenerator.generate();
		assertNotNull(classMapper);

		final IOrmCRUDQueryGenerator<Zoo_People> ormGenerator = new OrmCRUDQueryGenerator<Zoo_People>(new DefaultDialect(), classMapper);
		final IOrmCRUDQuery ormCrudQuery = ormGenerator.generate();
		assertNotNull(ormCrudQuery);

		System.out.println(ormCrudQuery.getBaseSelectClause());
		System.out.println(ormCrudQuery.getBaseSelectClause("alias."));
		System.out.println(ormCrudQuery.getBaseFromClause());

		assertEquals("ZOO.PEOPLE.FIRSTCLOB, ZOO.PEOPLE.ID, ZOO.PEOPLE.SECONDBLOB, ZOO.PEOPLE.BIRTHDATE, ZOO.PEOPLE.LASTNAME, ZOO.PEOPLE.FIRSTNAME, ZOO.PEOPLE.FIRSTBLOB, ZOO.PEOPLE.DEATHDATE", ormCrudQuery.getBaseSelectClause());
		assertEquals("alias.FIRSTCLOB, alias.ID, alias.SECONDBLOB, alias.BIRTHDATE, alias.LASTNAME, alias.FIRSTNAME, alias.FIRSTBLOB, alias.DEATHDATE", ormCrudQuery.getBaseSelectClause("alias."));
		assertEquals("ZOO.PEOPLE", ormCrudQuery.getBaseFromClause());

	}
}
